@@ -60,7 +60,10 @@ static inline void kseq2bseq1(const kseq_t *ks, bseq1_t *s, int conversion, int
 	s->alignment_score = 0;
 	if (conversion){
 		s->oseq = strdup(ks->seq.s);
-		bsConversion(s->seq, s->subsitution_pattern, s->l_seq); 
+		bsConversion(s->seq, s->subsitution_pattern, s->l_seq);
+		for (int i = 0; i < s->l_seq; ++i){
+			s->oseq[i] = s->oseq[i] < 4 ? s->oseq[i] : nst_nt4_table[(int)s->oseq[i]];
+		}
 	}
 	else{
 		s->oseq = NULL;
@@ -162,19 +165,40 @@ void bwa_fill_scmat(int a, int b, int8_t mat[25])
 	for (j = 0; j < 5; ++j) mat[k++] = -1;
 }
 
+char getMethylationContext(int base1, int base2, int base3, int meth_status, int *cg_count, int *ch_count){
+	if (base1 == 1 && base2 == 2){
+		*cg_count += 1;
+		if (meth_status) return 'X';
+		return 'x';
+	}
+	else if (base1 == 1 && base3 == 2){
+		*ch_count += 1;
+		if (meth_status) return 'Y';
+		return 'y';
+	}
+	else{
+		*ch_count += 1;
+		if (meth_status) return 'Z';
+    	return 'z';
+	}
+}
+
 // Generate CIGAR when the alignment end points are known
-uint32_t *bwa_gen_cigar2(const int8_t mat[25], int o_del, int e_del, int o_ins, int e_ins, int w_, int64_t l_pac, const uint8_t *pac, int l_query, uint8_t *query, int64_t rb, int64_t re, int *score, int *n_cigar, int *NM)
+uint32_t *bwa_gen_cigar2(const int8_t mat[25], int o_del, int e_del, int o_ins, int e_ins, int w_, int64_t l_pac, const uint8_t *pac, const uint8_t *opac, int l_query, 
+                         uint8_t *query, int64_t rb, int64_t re, int *score, int *n_cigar, int *NM, 
+                         int64_t crick_l, uint8_t *oquery, int *cg_meth, int *cg_unmeth, int *ch_meth, int *ch_unmeth)
 {
 	uint32_t *cigar = 0;
 	uint8_t tmp, *rseq;
-	int i;
+	int i, reverse;
 	int64_t rlen;
 	kstring_t str;
 	const char *int2base;
-
 	if (n_cigar) *n_cigar = 0;
 	if (NM) *NM = -1;
 	if (l_query <= 0 || rb >= re || (rb < l_pac && re > l_pac)) return 0; // reject if negative length or bridging the forward and reverse strand
+	reverse = rb > l_pac ? 1: 0; 
+
 	rseq = bns_get_seq(l_pac, pac, rb, re, &rlen);
 	if (re - rb != rlen) goto ret_gen_cigar; // possible if out of range
 	if (rb >= l_pac) { // then reverse both query and rseq; this is to ensure indels to be placed at the leftmost position
@@ -211,8 +235,33 @@ uint32_t *bwa_gen_cigar2(const int8_t mat[25], int o_del, int e_del, int o_ins,
 		}
 		*score = ksw_global2(l_query, query, rlen, rseq, 5, mat, o_del, e_del, o_ins, e_ins, w, n_cigar, &cigar);
 	}
-	if (NM && n_cigar) {// compute NM and MD
+	if (n_cigar) {// compute NM and MD
+		int is_crick, ref_base, methyl_base;
+		char meth_state;
+		uint8_t *cseq;
+		int64_t clen;
+		// check crick status, crick pac len is 2/4 and 4/4 of pac 
+		is_crick = 0;
+		if (rb >= crick_l && rb < l_pac) is_crick = 1;
+		else if (rb >= crick_l + l_pac) is_crick = 1;
+
+		// set reference base by mapping strand
+		ref_base = is_crick? 2: 1;
+		methyl_base = is_crick? 0: 3; 
+
+		if (rb > re) cseq = bns_get_seq(l_pac, opac, rb + 2, re - 2, &clen);
+		else cseq = bns_get_seq(l_pac, opac, rb - 2, re + 2, &clen);
+		if (rb >= l_pac) { // then reverse both query and rseq; this is to ensure indels to be placed at the leftmost position
+			for (i = 0; i < clen>>1; ++i)
+				tmp = cseq[i], cseq[i] = cseq[clen - 1 - i], cseq[clen - 1 - i] = tmp;
+			for (i = 0; i < l_query>>1; ++i)
+				tmp = oquery[i], oquery[i] = oquery[l_query - 1 - i], oquery[l_query - 1 - i] = tmp;
+		}
 		int k, x, y, u, n_mm = 0, n_gap = 0;
+		int base1, base2, base3, meth_pos = 0;
+		int ch_meth_count = 0, ch_unmeth_count = 0, cg_meth_count = 0, cg_unmeth_count = 0;
+		kstring_t meth_str;
+		meth_str.l = meth_str.m = 0; meth_str.s = 0;
 		str.l = str.m = *n_cigar * 4; str.s = (char*)cigar; // append MD to CIGAR
 		int2base = rb < l_pac? "ACGTN" : "TGCAN";
 		for (k = 0, x = y = u = 0; k < *n_cigar; ++k) {
@@ -221,26 +270,63 @@ uint32_t *bwa_gen_cigar2(const int8_t mat[25], int o_del, int e_del, int o_ins,
 			op  = cigar[k]&0xf, len = cigar[k]>>4;
 			if (op == 0) { // match
 				for (i = 0; i < len; ++i) {
-					if (query[x + i] != rseq[y + i]) {
+					if (!cseq) break;
+					if (cseq[y + i + 2] == ref_base){
+						// select bases to get relative methylation context
+						if (!reverse && !is_crick) base1 = cseq[y + i + 2], base2 = cseq[y +i + 3], base3 = cseq[y + i + 4]; // wc2t
+						else if (!reverse && is_crick) base1 = cseq[y + i + 1], base2 = cseq[y + i + 2], base3 = cseq[y + i + 3]; // cg2a
+						else if (reverse && !is_crick) base1 = cseq[y + i + 2], base2 = cseq[y +i + 1], base3 = cseq[y + i]; // wg2a
+						else base1 = cseq[y + i + 3], base2 = cseq[y + i + 2], base3 = cseq[y + i + 1]; // cc2t
+						// check methylation status
+						if (oquery[x + i] == ref_base) {
+							meth_state = getMethylationContext(base1, base2, base3, 1, &cg_meth_count, &ch_meth_count);
+							if (meth_pos > 0) kputw(meth_pos, &meth_str);
+							kputc(meth_state, &meth_str);
+							meth_pos = 0, ++u;
+						}
+						else if (oquery[x + i] == methyl_base) {
+							meth_state = getMethylationContext(base1, base2, base3, 0, &cg_unmeth_count, &ch_unmeth_count);
+							if (meth_pos > 0) kputw(meth_pos, &meth_str);
+							kputc(meth_state, &meth_str);
+							meth_pos = 0, ++u;
+						}
+						else {
+							kputw(u, &str);
+							kputc(int2base[cseq[ y+ i +2]], &str);
+							++n_mm; ++meth_pos, u = 0;
+						}
+					}
+					else if (oquery[x + i] != cseq[y + i +2]) {
 						kputw(u, &str);
-						kputc(int2base[rseq[y+i]], &str);
-						++n_mm; u = 0;
-					} else ++u;
+						kputc(int2base[cseq[ y+ i +2]], &str);
+						++n_mm; ++meth_pos, u = 0;
+					} else ++u, ++meth_pos;
 				}
 				x += len; y += len;
 			} else if (op == 2) { // deletion
 				if (k > 0 && k < *n_cigar - 1) { // don't do the following if D is the first or the last CIGAR
 					kputw(u, &str); kputc('^', &str);
 					for (i = 0; i < len; ++i)
-						kputc(int2base[rseq[y+i]], &str);
+						kputc(int2base[cseq[y+i+2]], &str);
 					u = 0; n_gap += len;
 				}
 				y += len;
-			} else if (op == 1) x += len, n_gap += len; // insertion
+			} else if (op == 1) x += len, n_gap += len, meth_pos += len; // insertion
 		}
-		kputw(u, &str); kputc(0, &str);
+		kputw(u, &str); 
+		if (meth_pos > 0) kputw(meth_pos, &meth_str); 
+		kputsn("\tXB:Z:", 6, &str);
+		kputsn(meth_str.s, meth_str.l, &str);
+		kputc(0, &str);
 		*NM = n_mm + n_gap;
 		cigar = (uint32_t*)str.s;
+		if (rb >= l_pac) // reverse back query
+			for (i = 0; i < l_query>>1; ++i)
+				tmp = oquery[i], oquery[i] = oquery[l_query - 1 - i], oquery[l_query - 1 - i] = tmp;
+		*cg_meth = cg_meth_count; *cg_unmeth = cg_unmeth_count;
+		*ch_meth = ch_meth_count; *ch_unmeth = ch_unmeth_count;
+		free(meth_str.s);
+		free(cseq);
 	}
 	if (rb >= l_pac) // reverse back query
 		for (i = 0; i < l_query>>1; ++i)
@@ -251,9 +337,10 @@ ret_gen_cigar:
 	return cigar;
 }
 
-uint32_t *bwa_gen_cigar(const int8_t mat[25], int q, int r, int w_, int64_t l_pac, const uint8_t *pac, int l_query, uint8_t *query, int64_t rb, int64_t re, int *score, int *n_cigar, int *NM)
+uint32_t *bwa_gen_cigar(const int8_t mat[25], int q, int r, int w_, int64_t l_pac, const uint8_t *pac, const uint8_t *opac, int l_query, uint8_t *query, int64_t rb, int64_t re, int *score, int *n_cigar, int *NM, 
+                        int64_t crick_l, uint8_t *oquery, int *cg_meth, int *cg_unmeth, int *ch_meth, int *ch_unmeth)
 {
-	return bwa_gen_cigar2(mat, q, r, q, r, w_, l_pac, pac, l_query, query, rb, re, score, n_cigar, NM);
+	return bwa_gen_cigar2(mat, q, r, q, r, w_, l_pac, pac, opac, l_query, query, rb, re, score, n_cigar, NM, crick_l, oquery, cg_meth, cg_unmeth, ch_meth, ch_unmeth);
 }
 
 /*********************
@@ -327,6 +414,10 @@ bwaidx_t *bwa_idx_load_from_disk(const char *hint, int which)
 			err_fread_noeof(idx->pac, 1, idx->bns->l_pac/4+1, idx->bns->fp_pac); // concatenated 2-bit encoded sequence
 			err_fclose(idx->bns->fp_pac);
 			idx->bns->fp_pac = 0;
+			idx->opac = calloc(idx->bns->l_pac/4+1, 1);
+			err_fread_noeof(idx->opac, 1, idx->bns->l_pac/4+1, idx->bns->fp_opac); // concatenated 2-bit encoded sequence
+			err_fclose(idx->bns->fp_opac);
+			idx->bns->fp_opac = 0;
 		}
 	}
 	free(prefix);
@@ -345,6 +436,7 @@ void bwa_idx_destroy(bwaidx_t *idx)
 		if (idx->bwt) bwt_destroy(idx->bwt);
 		if (idx->bns) bns_destroy(idx->bns);
 		if (idx->pac) free(idx->pac);
+		if (idx->opac) free(idx->opac);
 	} else {
 		free(idx->bwt); free(idx->bns->anns); free(idx->bns);
 		if (!idx->is_shm) free(idx->mem);
