@@ -60,9 +60,7 @@ mem_opt_t *mem_opt_init()
 	o->zdrop = 100;
 	o->pen_unpaired = 17;
 	o->pen_clip5 = o->pen_clip3 = 5;
-
 	o->max_mem_intv = 20;
-
 	o->min_seed_len = 19;
 	o->split_width = 10;
 	o->max_occ = 500;
@@ -70,7 +68,7 @@ mem_opt_t *mem_opt_init()
 	o->max_ins = 10000;
 	o->mask_level = 0.50;
 	o->drop_ratio = 0.50;
-	o->XA_drop_ratio = 0.80;
+	o->XA_drop_ratio = 0.95;
 	o->split_factor = 1.5;
 	o->chunk_size = 10000000;
 	o->n_threads = 1;
@@ -82,6 +80,8 @@ mem_opt_t *mem_opt_init()
 	o->max_chain_extend = 1<<30;
 	o->mapQ_coef_len = 50; o->mapQ_coef_fac = log(o->mapQ_coef_len);
 	o->undirectional = 0;
+	o->ch_conversion_threshold = 5;
+	o->ch_conversion_proportion = 0.5;
 	bwa_fill_scmat(o->a, o->b, o->mat);
 	return o;
 }
@@ -428,7 +428,7 @@ int mem_patch_reg(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac,
 	w += a->w + b->w;
 	w = w < opt->w<<2? w : opt->w<<2;
 	if (bwa_verbose >= 4) printf("* test potential hit merge with global alignment; w=%d\n", w);
-	bwa_gen_cigar2(opt->mat, opt->o_del, opt->e_del, opt->o_ins, opt->e_ins, w, bns->l_pac, pac, b->qe - a->qb, query + a->qb, a->rb, b->re, &score, 0, 0);
+	bwa_gen_cigar2(opt->mat, opt->o_del, opt->e_del, opt->o_ins, opt->e_ins, w, bns->l_pac, pac, pac, b->qe - a->qb, query + a->qb, a->rb, b->re, &score, 0, 0, bns->crick_l, 0, 0, 0, 0, 0);
 	q_s = (int)((double)(b->qe - a->qb) / ((b->qe - b->qb) + (a->qe - a->qb)) * (b->score + a->score) + .499); // predicted score from query
 	r_s = (int)((double)(b->re - a->rb) / ((b->re - b->rb) + (a->re - a->rb)) * (b->score + a->score) + .499); // predicted score from ref
 	if (bwa_verbose >= 4) printf("* score=%d;(%d,%d)\n", score, q_s, r_s);
@@ -829,31 +829,32 @@ void mem_aln2sam(const mem_opt_t *opt, const bntseq_t *bns, kstring_t *str, bseq
 {
 	int i, l_name;
 	mem_aln_t ptmp = list[which], *p = &ptmp, mtmp, *m = 0; // make a copy of the alignment to convert
-
 	if (m_) mtmp = *m_, m = &mtmp;
 	// set flag
 	int is_mate_crick = 0;
 	int is_crick = 0;
 	int bs_conflict = 0;
+	int reverse = 0;
 	int bs_mate_conflict = 0;
+	double ch_meth = 0, ch_unmeth = 0;
 	if (p->rid >= 0){
 		is_crick = bns->anns[p->rid].is_crick;
+		reverse = p->is_rev ? 1: 0;
+		ch_meth += p->ch_meth;
+		ch_unmeth += p->ch_unmeth;
 		if (p->XA){
  			bs_conflict = countAlts(p->XA, is_crick);
 		}
 		if (is_crick){
-			p->pos = getCrickMappingPos(p->cigar, p->n_cigar, bns->anns[p->rid].len, p->pos);
-			reverseCigar(p->cigar, p->n_cigar);
-			p->is_rev = formatSeqQual(is_crick, s->oseq, s->qual, p->is_rev);	
+			p->is_rev = is_crick ? 1 : 0;	
 			if (m && m->rid >= 0){
+				ch_unmeth += m->ch_unmeth;
+				ch_meth += m->ch_meth;
 				is_mate_crick = bns->anns[m->rid].is_crick;
 				if (m->XA){
  					bs_mate_conflict = countAlts(m->XA, is_crick);
 				}
-				if (is_mate_crick){
-					m->pos = getCrickMappingPos(m->cigar, m->n_cigar, bns->anns[m->rid].len, m->pos);
-					m->is_rev = 1;
-				}
+				m->is_rev = is_mate_crick ? 1: 0;
 				if (is_crick != is_mate_crick){
 					bs_conflict = 1;
 					bs_mate_conflict = 1;
@@ -861,11 +862,9 @@ void mem_aln2sam(const mem_opt_t *opt, const bntseq_t *bns, kstring_t *str, bseq
 			}
 		}
 		else{
-			p->is_rev = formatSeqQual(is_crick, s->oseq, s->qual, p->is_rev);
+			p->is_rev = is_crick ? 1 : 0;
 			if (m && m->rid >= 0){
-				if (! is_mate_crick){
-					m->is_rev = 0;	
-				}
+				m->is_rev = is_mate_crick ? 1 : 0;
 			}
 		}
 	}
@@ -932,34 +931,49 @@ void mem_aln2sam(const mem_opt_t *opt, const bntseq_t *bns, kstring_t *str, bseq
 		} else kputc('0', str);
 	} else kputsn("*\t0\t0", 5, str);
 	kputc('\t', str);
-
 	// print SEQ and QUAL
 	if (p->flag & 0x100) { // for secondary alignments, don't write SEQ and QUAL
 		kputsn("*\t*", 3, str);
-	} else { // all strands
+	} else if (!reverse) { // the forward strand
 		int i, qb = 0, qe = s->l_seq;
 		if (p->n_cigar && which && !(opt->flag&MEM_F_SOFTCLIP) && !p->is_alt) { // have cigar && not the primary alignment && not softclip all
 			if ((p->cigar[0]&0xf) == 4 || (p->cigar[0]&0xf) == 3) qb += p->cigar[0]>>4;
 			if ((p->cigar[p->n_cigar-1]&0xf) == 4 || (p->cigar[p->n_cigar-1]&0xf) == 3) qe -= p->cigar[p->n_cigar-1]>>4;
 		}
 		ks_resize(str, str->l + (qe - qb) + 1);
-		// for (i = qb; i < qe; ++i) str->s[str->l++] = "ACGTN"[(int)s->seq[i]];
-		//int meta_loc = getConversionBase()
-        for (i = qb; i < qe; ++i){
-			str->s[str->l++] = s->oseq[i];
-		} 
+		for (i = qb; i < qe; ++i) str->s[str->l++] = "ACGTN"[(int)s->oseq[i]];
 		kputc('\t', str);
 		if (s->qual) { // printf qual
 			ks_resize(str, str->l + (qe - qb) + 1);
 			for (i = qb; i < qe; ++i) str->s[str->l++] = s->qual[i];
 			str->s[str->l] = 0;
 		} else kputc('*', str);
-	} 
+	} else { // the reverse strand
+		int i, qb = 0, qe = s->l_seq;
+		if (p->n_cigar && which && !(opt->flag&MEM_F_SOFTCLIP) && !p->is_alt) {
+			if ((p->cigar[0]&0xf) == 4 || (p->cigar[0]&0xf) == 3) qe -= p->cigar[0]>>4;
+			if ((p->cigar[p->n_cigar-1]&0xf) == 4 || (p->cigar[p->n_cigar-1]&0xf) == 3) qb += p->cigar[p->n_cigar-1]>>4;
+		}
+		ks_resize(str, str->l + (qe - qb) + 1);
+		for (i = qe-1; i >= qb; --i) str->s[str->l++] = "TGCAN"[(int)s->oseq[i]];
+		kputc('\t', str);
+		if (s->qual) { // printf qual
+			ks_resize(str, str->l + (qe - qb) + 1);
+			for (i = qe-1; i >= qb; --i) str->s[str->l++] = s->qual[i];
+			str->s[str->l] = 0;
+		} else kputc('*', str);
+	}
 
 	// print optional tags
 	if (p->n_cigar) {
 		kputsn("\tNM:i:", 6, str); kputw(p->NM, str);
 		kputsn("\tMD:Z:", 6, str); kputs((char*)(p->cigar + p->n_cigar), str);
+		if (ch_unmeth + ch_meth >= opt->ch_conversion_threshold){
+			double conversion_prop = ch_meth / (ch_unmeth + ch_meth);
+			kputsn("\tXC:i:", 6, str);
+			if (conversion_prop < opt->ch_conversion_proportion) kputw(0, str);
+			else kputw(1, str);
+		}
 	}
 	if (p->score >= 0) { kputsn("\tAS:i:", 6, str); kputw(p->score, str); }
 	if (p->sub >= 0) { kputsn("\tXS:i:", 6, str); kputw(p->sub, str); }
@@ -1088,16 +1102,15 @@ void mem_reorder_primary5(int T, mem_alnreg_v *a)
 }
 
 // TODO (future plan): group hits into a uint64_t[] array. This will be cleaner and more flexible
-void mem_reg2sam(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, bseq1_t *s, mem_alnreg_v *a, int extra_flag, const mem_aln_t *m)
+void mem_reg2sam(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac,  const uint8_t *opac, bseq1_t *s, mem_alnreg_v *a, int extra_flag, const mem_aln_t *m)
 {
-	extern char **mem_gen_alt(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, mem_alnreg_v *a, int l_query, const char *query);
+	extern char **mem_gen_alt(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, const uint8_t *opac, mem_alnreg_v *a, int l_query, const char *query, const char *oquery);
 	kstring_t str;
 	kvec_t(mem_aln_t) aa;
 	int k, l;
 	char **XA = 0;
-
 	if (!(opt->flag & MEM_F_ALL))
-		XA = mem_gen_alt(opt, bns, pac, a, s->l_seq, s->seq);
+		XA = mem_gen_alt(opt, bns, pac, opac, a, s->l_seq, s->seq, s->oseq);
 	kv_init(aa);
 	str.l = str.m = 0; str.s = 0;
 	for (k = l = 0; k < a->n; ++k) {
@@ -1107,7 +1120,7 @@ void mem_reg2sam(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac,
 		if (p->secondary >= 0 && (p->is_alt || !(opt->flag&MEM_F_ALL))) continue;
 		if (p->secondary >= 0 && p->secondary < INT_MAX && p->score < a->a[p->secondary].score * opt->drop_ratio) continue;
 		q = kv_pushp(mem_aln_t, aa);
-		*q = mem_reg2aln(opt, bns, pac, s->l_seq, s->seq, p);
+		*q = mem_reg2aln(opt, bns, pac, opac, s->l_seq, s->seq, p, s->oseq);
 		assert(q->rid >= 0); // this should not happen with the new code
 		q->XA = XA? XA[k] : 0;
 		q->flag |= extra_flag; // flag secondary
@@ -1120,7 +1133,7 @@ void mem_reg2sam(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac,
 	}
 	if (aa.n == 0) { // no alignments good enough; then write an unaligned record
 		mem_aln_t t;
-		t = mem_reg2aln(opt, bns, pac, s->l_seq, s->seq, 0);
+		t = mem_reg2aln(opt, bns, pac, opac, s->l_seq, s->seq, 0, s->oseq);
 		t.flag |= extra_flag;
 		mem_aln2sam(opt, bns, &str, s, 1, &t, 0, m);
 	} else {
@@ -1136,13 +1149,15 @@ void mem_reg2sam(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac,
 	}
 }
 
-mem_alnreg_v mem_align1_core(const mem_opt_t *opt, const bwt_t *bwt, const bntseq_t *bns, const uint8_t *pac, int l_seq, char *seq, void *buf)
+mem_alnreg_v mem_align1_core(const mem_opt_t *opt, const bwt_t *bwt, const bntseq_t *bns, const uint8_t *pac, 
+                             int l_seq, char *seq, void *buf)
 {
 	int i;
 	mem_chain_v chn;
 	mem_alnreg_v regs;
 
-	for (i = 0; i < l_seq; ++i) // convert to 2-bit encoding if we have not done so
+	// convert bisulfite and original seq to 2bit encoding 
+	for (i = 0; i < l_seq; ++i) 
 		seq[i] = seq[i] < 4? seq[i] : nst_nt4_table[(int)seq[i]];
 
 	chn = mem_chain(opt, bwt, bns, l_seq, (uint8_t*)seq, buf);
@@ -1174,12 +1189,12 @@ mem_alnreg_v mem_align1_core(const mem_opt_t *opt, const bwt_t *bwt, const bntse
 	return regs;
 }
 
-mem_aln_t mem_reg2aln(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, int l_query, const char *query_, const mem_alnreg_t *ar)
+mem_aln_t mem_reg2aln(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, const uint8_t *opac, int l_query, const char *query_, const mem_alnreg_t *ar, const char *oquery_)
 {
 	mem_aln_t a;
 	int i, w2, tmp, qb, qe, NM, score, is_rev, last_sc = -(1<<30), l_MD;
 	int64_t pos, rb, re;
-	uint8_t *query;
+	uint8_t *query, *oquery;
 
 	memset(&a, 0, sizeof(mem_aln_t));
 	if (ar == 0 || ar->rb < 0 || ar->re < 0) { // generate an unmapped record
@@ -1191,6 +1206,9 @@ mem_aln_t mem_reg2aln(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *
 	query = malloc(l_query);
 	for (i = 0; i < l_query; ++i) // convert to the nt4 encoding
 		query[i] = query_[i] < 5? query_[i] : nst_nt4_table[(int)query_[i]];
+	oquery = malloc(l_query);
+	for (i = 0; i < l_query; ++i) // convert to the nt4 encoding
+		oquery[i] = oquery_[i] < 5? oquery_[i] : nst_nt4_table[(int)oquery_[i]];
 	a.mapq = ar->secondary < 0? mem_approx_mapq_se(opt, ar) : 0;
 	if (ar->secondary >= 0) a.flag |= 0x100; // secondary alignment
 	tmp = infer_bw(qe - qb, re - rb, ar->truesc, opt->a, opt->o_del, opt->e_del);
@@ -1202,12 +1220,13 @@ mem_aln_t mem_reg2aln(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *
 	do {
 		free(a.cigar);
 		w2 = w2 < opt->w<<2? w2 : opt->w<<2;
-		a.cigar = bwa_gen_cigar2(opt->mat, opt->o_del, opt->e_del, opt->o_ins, opt->e_ins, w2, bns->l_pac, pac, qe - qb, (uint8_t*)&query[qb], rb, re, &score, &a.n_cigar, &NM);
+		a.cigar = bwa_gen_cigar2(opt->mat, opt->o_del, opt->e_del, opt->o_ins, opt->e_ins, w2, bns->l_pac, pac, opac, qe - qb, (uint8_t*)&query[qb], rb, re, &score, &a.n_cigar, &NM, bns->crick_l, oquery, &a.cg_meth, &a.cg_unmeth, &a.ch_meth, &a.ch_unmeth);
 		if (bwa_verbose >= 4) printf("* Final alignment: w2=%d, global_sc=%d, local_sc=%d\n", w2, score, ar->truesc);
 		if (score == last_sc || w2 == opt->w<<2) break; // it is possible that global alignment and local alignment give different scores
 		last_sc = score;
 		w2 <<= 1;
 	} while (++i < 3 && score < ar->truesc - opt->a);
+	// error thrown here
 	l_MD = strlen((char*)(a.cigar + a.n_cigar)) + 1;
 	a.NM = NM;
 	pos = bns_depos(bns, rb < bns->l_pac? rb : re - 1, &is_rev);
@@ -1243,6 +1262,7 @@ mem_aln_t mem_reg2aln(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *
 	a.score = ar->score; a.sub = ar->sub > ar->csub? ar->sub : ar->csub;
 	a.is_alt = ar->is_alt; a.alt_sc = ar->alt_sc;
 	free(query);
+	free(oquery);
 	return a;
 }
 
@@ -1251,6 +1271,7 @@ typedef struct {
 	const bwt_t *bwt;
 	const bntseq_t *bns;
 	const uint8_t *pac;
+	const uint8_t *opac;
 	const mem_pestat_t *pes;
 	smem_aux_t **aux;
 	bseq1_t *seqs;
@@ -1274,23 +1295,23 @@ static void worker1(void *data, int i, int tid)
 
 static void worker2(void *data, int i, int tid)
 {
-	extern int mem_sam_pe(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, const mem_pestat_t pes[4], uint64_t id, bseq1_t s[2], mem_alnreg_v a[2]);
+	extern int mem_sam_pe(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, const uint8_t *opac, const mem_pestat_t pes[4], uint64_t id, bseq1_t s[2], mem_alnreg_v a[2]);
 	extern void mem_reg2ovlp(const mem_opt_t *opt, const bntseq_t *bns, const uint8_t *pac, bseq1_t *s, mem_alnreg_v *a);
 	worker_t *w = (worker_t*)data;
 	if (!(w->opt->flag&MEM_F_PE)) {
 		if (bwa_verbose >= 4) printf("=====> Finalizing read '%s' <=====\n", w->seqs[i].name);
 		mem_mark_primary_se(w->opt, w->regs[i].n, w->regs[i].a, w->n_processed + i);
 		if (w->opt->flag & MEM_F_PRIMARY5) mem_reorder_primary5(w->opt->T, &w->regs[i]);
-		mem_reg2sam(w->opt, w->bns, w->pac, &w->seqs[i], &w->regs[i], 0, 0);
+		mem_reg2sam(w->opt, w->bns, w->pac, w->opac, &w->seqs[i], &w->regs[i], 0, 0);
 		free(w->regs[i].a);
 	} else {
 		if (bwa_verbose >= 4) printf("=====> Finalizing read pair '%s' <=====\n", w->seqs[i<<1|0].name);
-		mem_sam_pe(w->opt, w->bns, w->pac, w->pes, (w->n_processed>>1) + i, &w->seqs[i<<1], &w->regs[i<<1]);
+		mem_sam_pe(w->opt, w->bns, w->pac, w->opac, w->pes, (w->n_processed>>1) + i, &w->seqs[i<<1], &w->regs[i<<1]);
 		free(w->regs[i<<1|0].a); free(w->regs[i<<1|1].a);
 	}
 }
 
-void mem_process_seqs(const mem_opt_t *opt, const bwt_t *bwt, const bntseq_t *bns, const uint8_t *pac, int64_t n_processed, int n, bseq1_t *seqs, const mem_pestat_t *pes0)
+void mem_process_seqs(const mem_opt_t *opt, const bwt_t *bwt, const bntseq_t *bns, const uint8_t *pac, const uint8_t *opac, int64_t n_processed, int n, bseq1_t *seqs, const mem_pestat_t *pes0)
 {
 	extern void kt_for(int n_threads, void (*func)(void*,int,int), void *data, int n);
 	worker_t w;
@@ -1301,7 +1322,7 @@ void mem_process_seqs(const mem_opt_t *opt, const bwt_t *bwt, const bntseq_t *bn
 	ctime = cputime(); rtime = realtime();
 	global_bns = bns;
 	w.regs = malloc(n * sizeof(mem_alnreg_v));
-	w.opt = opt; w.bwt = bwt; w.bns = bns; w.pac = pac;
+	w.opt = opt; w.bwt = bwt; w.bns = bns; w.pac = pac; w.opac = opac;
 	w.seqs = seqs; w.n_processed = n_processed;
 	w.pes = &pes[0];
 	w.aux = malloc(opt->n_threads * sizeof(smem_aux_t));
