@@ -12,10 +12,11 @@ class IndexOutput:
 
     """
 
-    def __init__(self, genome_database: str = None):
+    def __init__(self, genome_database: str = None, block_size: int = None):
         # format genome_database path
         bwa_path, _ = get_external_paths()
         self.genome_database = self.generate_genome_directory(genome_database)
+        self.block_size = block_size
         self.bwa_path = bwa_path
         # set output object
         self.database_output = open(f'{self.genome_database}BSB_ref.fa', 'w')
@@ -42,12 +43,7 @@ class IndexOutput:
         """
         # write forward bisulfite converted sequence
         self.database_output.write(f'>{contig_id}\n')
-        self.database_output.write(f'{contig_sequence.replace("C", "T").replace("c", "t")}\n')
-
-        # write reverse bisulfite converted sequence
-        reverse_contig_sequence = reverse_complement(contig_sequence)
-        self.database_output.write(f'>{contig_id}_crick_bs\n')
-        self.database_output.write(f'{reverse_contig_sequence.replace("C", "T").replace("c", "t")}\n')
+        self.database_output.write(f'{contig_sequence}\n')
 
     def build_index(self):
         """Launch external commands for 4 processed reference files and collect external stdout for log file
@@ -55,7 +51,7 @@ class IndexOutput:
         # format output and input
         ref_file = f'{self.genome_database}BSB_ref.fa'
         # collect external command
-        indx_command = [f'{self.bwa_path}', 'index', '-a', 'bwtsw', ref_file]
+        indx_command = [f'{self.bwa_path}', 'index', '-a', 'bwtsw', '-b', f'{self.block_size}', ref_file]
         # run external command
         subprocess.run(args=indx_command)
 
